# -*- coding: utf-8 -*-
#
# Copyright (C) 2009 Jeff Hammel <jhammel@openplans.org>
# Copyright (C) 2013 Ryan J Ollos <ryan.j.ollos@gmail.com>
# Copyright (C) 2014 Giuseppe Ursino <giursino@gmail.com>
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution.
#

import os
import shutil

from trac.config import Option
from trac.core import Component
from trac.env import open_environment
from trac.perm import PermissionSystem
from trac.ticket import Ticket

from tracsqlhelper import get_all_dict
from tracsqlhelper import insert_row_from_dict
from tracsqlhelper import get_column

from trac.attachment import Attachment


class TicketMover(Component):

    permission = Option('ticket', 'move_permission', 'TICKET_ADMIN',
                        """permission needed to move tickets between
                           Trac projects""")

    ### Internal methods

    def projects(self, user):
        base_path, _project = os.path.split(self.env.path)
        _projects = [p for p in os.listdir(base_path)
                     if p != _project]
        projects = {}
        for project in _projects:
            path = os.path.join(base_path, project)
            try:
                env = open_environment(path, use_cache=True)
            except:
                continue
            perm = PermissionSystem(env)
            if self.permission in perm.get_user_permissions(user):
                projects[project] = env

        return projects
        
    def move(self, ticket_id, author, dst_env_name, delete=False, keepstatus=False):
        """
        move a ticket to another environment
        
        dst_env_name: environment to move to
        """
        
        # source environment
        src_env = self.env

        # open the environment if it is a string
        if isinstance(dst_env_name, basestring):
            base_path, _project = os.path.split(src_env.path)
            dst_env = open_environment(os.path.join(base_path, dst_env_name), use_cache=True)

        # get the old ticket
        src_ticket = Ticket(src_env, ticket_id)

        # make a new ticket from the old ticket values
        dst_ticket = Ticket(dst_env)
        dst_ticket.values = src_ticket.values.copy()
        dst_ticket.insert(when=src_ticket.time_created)

        # copy the changelog DBs
        table = 'ticket_change'
        _id	= 'ticket'
        for row in get_all_dict(src_env,
                                "SELECT * FROM %s WHERE %s=%%s"
                                % (table, _id), str(src_ticket.id)):
            row[_id] = dst_ticket.id
            insert_row_from_dict(dst_env, table, row)
        
        # copy the attachment DBs
        table = 'attachment'
        _id	= 'id'
        filenames = []
        for row in get_all_dict(src_env,
                                "SELECT * FROM %s WHERE type = 'ticket' AND %s = %%s" % 
                                (table , _id), str(src_ticket.id)):
            row[_id] = dst_ticket.id
            insert_row_from_dict(dst_env, table, row)
            # remember file name...
            filenames.append(row['filename'])
        
        
        # copy the attachments
        for filename in filenames:
            src_path_file = Attachment._get_path(src_env.path , 'ticket' , str(src_ticket.id) , filename)
            dst_path_file = Attachment._get_path(dst_env.path , 'ticket' , str(dst_ticket.id) , filename)     
            
            dst_path , dst_file = os.path.split(dst_path_file)  						   
            
            if os.path.isfile(src_path_file):
                if not os.path.exists(dst_path):
                     os.makedirs(dst_path)
                
                shutil.copyfile(src_path_file , dst_path_file)
                self.log.info('attachments copy "%s" -> "%s"...' % (src_path_file , dst_path_file));
        
        # note the previous location on the new ticket
        from_url = src_env.abs_href('ticket', src_ticket.id)
        dst_ticket.save_changes(author, 'moved from %s' % from_url)
        self.log.info('moved from=%s ', from_url);
        
        
        # location of new ticket
        if dst_env.config['trac'].get('base_url'):
            new_location = dst_env.abs_href('ticket', dst_ticket.id)
        else:
            base_path, _project = os.path.split(src_env.path)
            base_url = from_url[:from_url.find(_project)]
            new_location= "%s%s/ticket/%s" % (base_url, dst_env_name, dst_ticket.id)
        
        
        if delete:
            src_ticket.delete()
        else:
            # close old ticket and point to new one
            src_ticket['status'] = u'closed'
            src_ticket['resolution'] = u'moved'
            src_ticket.save_changes(author, u'moved to %s' % new_location)
            self.log.info('moved to=%s ', new_location );
            
        
        save_dst_ticket = False
        
        # status of the imported ticket will be 'new' if it was not flagged to keep the current status
        if not keepstatus:
            dst_ticket['status']= u'new'
            save_dst_ticket = True
        
        # check if the ticket fields exists on destination environment
        warnings = []
        
        # fields that have db table
        fields_to_check = ['component', 'version', 'milestone']
        for field in fields_to_check:
            try:
                src_field = dst_ticket.values[field]
                if not src_field: break
                dst_fields= get_column(dst_env, field, 'name')
                
                if src_field not in dst_fields:
                    dst_ticket[field]=u''
                    warnings.append(field)
            except Exception:
                pass
        
        # fields that are in db enum table
        fields_to_check_enumtable = ['priority']
        for field in fields_to_check_enumtable:
            try:
                src_field = dst_ticket.values[field]
                if not src_field: break
                dst_fields= get_column(dst_env, 'enum', 'name', 'type = \'%s\'' % (field))
                
                if src_field not in dst_fields:
                    dst_ticket[field]=u''
                    warnings.append(field)
            except Exception:
                pass
        
        # check ticket 'type' field
        try:
            src_field = dst_ticket.values['type']
            if not src_field: break
            dst_fields= get_column(dst_env, 'enum', 'name', 'type = \'ticket_type\'')
        
            if src_field not in dst_fields:
                dst_ticket['type']=u''
                warnings.append(field)
        except Exception:
            pass
        
        
        
        if len(warnings):
            save_dst_ticket = True
        
        # save ticket
        if save_dst_ticket:
            dst_ticket.save_changes(author)


        
        return (new_location, warnings)
